unit Mabcsave;

interface

uses
   SysUtils, WinTypes, WinProcs, Classes, Graphics, Controls, Forms,
   MabSave, IniFiles;

type
   TMabControlProperty = (cpPosition, cpSize, cpVisible);
   TMabControlProperties = set of TMabControlProperty;
   TMabControlSave = class(TMabSave)
   private
      { Private declarations }
      { Private declarations }
      FControl: TControl;
      FRemember: TMabControlProperties;
   protected
      { Protected declarations }
      procedure Notification(AComponent: TComponent; Operation: TOperation); override;
   public
      { Public declarations }
      constructor Create(AOwner: TComponent); override;

      procedure Save;
      procedure Load;
   published
      { Published declarations }
      property Control: TControl read FControl write FControl;
      property Filename;
      property Remember: TMabControlProperties read FRemember write FRemember;
   end;

implementation

constructor TMabControlSave.Create(AOwner: TComponent);
begin
   inherited Create(AOwner);

   FRemember := [cpPosition, cpSize, cpVisible];
end;

procedure TMabControlSave.Notification(AComponent: TComponent; Operation: TOperation);
begin
   if (Operation = opRemove) and (AComponent = Control) then
      Control := nil;
end;

procedure TMabControlSave.Save;
var
   PName, PMesg: PChar;
   Section: string;
begin
   try
      if IniFile = nil then
         raise Exception.Create('Filename must be have a value');
      if Control = nil then
         raise Exception.Create('Control must be specified');

      with Control do
      begin
         Section := Owner.ClassName + '.' + Name;
         if cpPosition in Remember then
         begin
            IniFile.WriteInteger(Section, 'Top', Top);
            IniFile.WriteInteger(Section, 'Left', Left);
         end;

         if cpSize in Remember then
         begin
            IniFile.WriteInteger(Section, 'Height', Height);
            IniFile.WriteInteger(Section, 'Width', Width);
         end;

         if cpVisible in Remember then
         begin
            IniFile.WriteBool(Section, 'Visible', Visible);
         end;
      end;
   except
      on E: Exception do
      begin
         PName := StrAlloc(Length(Name) + 1);
         PMesg := StrAlloc(Length(E.Message) + 1);
         StrPCopy(PName, Name);
         StrPCopy(PMesg, E.Message);
         Application.MessageBox(PMesg, PName, MB_ICONEXCLAMATION);
         StrDispose(PName);
         StrDispose(PMesg);
      end;
   end;
end;

procedure TMabControlSave.Load;
var
   PName, PMesg: PChar;
   Section: string;
   AStyle: TFontStyles;
begin
   try
      if IniFile = nil then
         raise Exception.Create('Filename must be have a value');
      if Control = nil then
         raise Exception.Create('Control must be specified');

      with Control do
      begin
         Section := Owner.ClassName + '.' + Name;
         if cpPosition in Remember then
         begin
            Top := IniFile.ReadInteger(Section, 'Top', Top);
            Left := IniFile.ReadInteger(Section, 'Left', Left);
         end;

         if cpSize in Remember then
         begin
            Height := IniFile.ReadInteger(Section, 'Height', Height);
            Width := IniFile.ReadInteger(Section, 'Width', Width);
         end;

         if cpVisible in Remember then
         begin
            Visible := IniFile.ReadBool(Section, 'Visible', Visible);
         end;
      end;
   except
      on E: Exception do
      begin
         PName := StrAlloc(Length(Name) + 1);
         PMesg := StrAlloc(Length(E.Message) + 1);
         StrPCopy(PName, Name);
         StrPCopy(PMesg, E.Message);
         Application.MessageBox(PMesg, PName, MB_ICONEXCLAMATION);
         StrDispose(PName);
         StrDispose(PMesg);
      end;
   end;
end;

end.end.
