unit Mabrup;

interface

uses
  SysUtils, WinTypes, WinProcs, Messages, Classes, Graphics, Controls,
  Forms, Dialogs, ExtCtrls, Menus;

type
  TMabRollUp = class(TCustomControl)
  private
    { Private declarations }
    SavedHeight: Integer;

    procedure DrawCaption;
  protected
    { Protected declarations }
    FCaption: TCaption;
    FCaptionHeight: Byte;
    FBorderWidth: Byte;
    FRolledUp: Boolean;

    procedure SetCaption(ACaption: TCaption);
    procedure SetCaptionHeight(ACaptionHeight: Byte);
    procedure SetBorderWidth(ABorderWidth: Byte);
    procedure SetRolledUp(ARolledUp: Boolean);

    procedure Paint; override;
    procedure WMLButtonUp(var msg: TWMLButtonUp); message WM_LBUTTONUP;
  public
    { Public declarations }
    constructor Create(AOwner: TComponent); override;
  published
    { Published declarations }
    property BorderWidth: Byte read FBorderWidth write SetBorderWidth;
    property Caption: TCaption read FCaption write SetCaption;
    property CaptionHeight: Byte read FCaptionHeight write SetCaptionHeight;
    property RolledUp: Boolean read FRolledUp write SetRolledUp;
    { republished properties }
    property Font;
    property ParentFont;
    property ParentShowHint;
    property ShowHint;
    property Visible;
    property OnDragDrop;
    property OnDragOver;
    property PopupMenu;
    property OnMouseMove;
  end;

implementation

{$R mabrup.res}

constructor TMabRollUp.Create(AOwner: TComponent);
begin
   inherited Create(AOwner);

	ControlStyle := [csAcceptsControls, csClickEvents, csSetCaption];
   FBorderWidth := GetSystemMetrics(SM_CYDLGFRAME);
   FCaptionHeight := GetSystemMetrics(SM_CYCAPTION);
   FRolledUp := False;

   SetBounds(Left, Top, 128, 128);
end;

procedure TMabRollUp.SetCaption(ACaption: TCaption);
begin
   if ACaption <> FCaption then
   begin
      FCaption := ACaption;
      DrawCaption;
   end;
end;

procedure TMabRollUp.SetCaptionHeight(ACaptionHeight: Byte);
begin
   if ACaptionHeight <> FCaptionHeight then
   begin
      FCaptionHeight := ACaptionHeight;
      Refresh;
   end;
end;

procedure TMabRollUp.SetBorderWidth(ABorderWidth: Byte);
begin
   if ABorderWidth <> FBorderWidth then
   begin
      FBorderWidth := ABorderWidth;
      Refresh;
   end;
end;

procedure TMabRollUp.SetRolledUp(ARolledUp: Boolean);
var
   i: Integer;
begin
   if ARolledUp <> FRolledUp then
   begin
      FRolledUp := ARolledUp;
      for i := 0 to ControlCount - 1 do
         Controls[i].Enabled := not RolledUp;

      if FRolledUp then
      begin
         SavedHeight := Height;
         Height := CaptionHeight + 2 * BorderWidth;
      end
      else
         Height := SavedHeight;
   end;
end;

procedure TMabRollUp.DrawCaption;
var
   BtnBmp: TBitmap;
   TempCaption: array[0..255] of Char;
   rc: TRect;
begin
   with Canvas do
   begin
      {
         Fill the entire caption with the inactive caption color,
         and draw an indented frame around it.
      }
      rc := Rect(BorderWidth, BorderWidth,
             Width - BorderWidth, CaptionHeight + BorderWidth);
      Brush.Color := clInactiveCaption;
      FillRect(rc);
      Frame3d(Canvas, rc, clBtnShadow, clBtnHighlight, 1);

      {
         Write the caption text in the active caption font color,
         clipping inside the valid area.
      }
      with rc do
      begin
         Left := Left + 1;
         Top := Top + 1;
         Right := Right - CaptionHeight - 1;
         Bottom := Bottom - 1;
      end;
      Font.Assign(Self.Font);
      {Font.Color := clCaptionText;}
      StrPCopy(TempCaption, Caption);
      DrawText(Handle, TempCaption, -1, rc, DT_LEFT + DT_SINGLELINE + DT_VCENTER);

      {
         Draw the 'roll up' button.
      }
      Brush.Color := clBtnFace;
      rc := Rect(Width - BorderWidth - CaptionHeight + 2, BorderWidth + 2,
            Width - BorderWidth - 2, BorderWidth + CaptionHeight - 2);
      FillRect(rc);
      Frame3d(Canvas, rc, clBtnHighlight, clBtnShadow, 1);

      BtnBmp := TBitmap.Create;
      try
         if RolledUp then
            BtnBmp.Handle := LoadBitmap(HInstance, 'UNROLL')
         else
            BtnBmp.Handle := LoadBitmap(HInstance, 'ROLLUP');

         BrushCopy(rc, BtnBmp, BtnBmp.Canvas.ClipRect,
            ColorToRGB(BtnBmp.Canvas.Pixels[0,0]));
      finally
         BtnBmp.Free;
      end;
   end;
end;

procedure TMabRollUp.Paint;
var
   rc: TRect;
begin
   inherited Paint;

   rc := Rect(0, 0, Width, Height);
   Canvas.Brush.Color := clBtnFace;
   Canvas.FillRect(rc);
   Frame3d(Canvas, rc, clBtnHighlight, clBtnShadow, 1);

   DrawCaption;
   {
	if RolledUp and (csDesigning in ComponentState) then
      with (Parent as TCustomControl).Canvas do
      begin
         Brush.Style := bsClear;
         Pen.Color := clWindowFrame;
         Pen.Style := psDash;
         Rectangle(0, 0, Self.Width, Self.SavedHeight);
         Pen.Style := psSolid;
         Brush.Style := bsSolid;
      end;
   }
   {
   rc := Rect(BorderWidth, CaptionHeight + BorderWidth * 2,
      Width - BorderWidth, Height - BorderWidth);
   Frame3d(Canvas, rc, clBtnShadow, clBtnHighlight, 1);
   }
end;

procedure TMabRollUp.WMLButtonUp(var msg: TWMLButtonUp);
var
   rc: TRect;
begin
   if Visible and (Msg.Keys = 0) then
   begin
      rc := Rect(Width - BorderWidth - CaptionHeight + 2, BorderWidth + 2,
            Width - BorderWidth - 2, BorderWidth + CaptionHeight - 2);

      if PtInRect(rc, Msg.Pos) then
         RolledUp := not RolledUp;
      BringToFront;
   end;
   Msg.Result := 0;
end;

end.
